import {Component} from '@angular/core';
import {BackendService, Priority, ScrumTask, ScrumUserstory} from '../services/backend.service';
import {NgbModal} from '@ng-bootstrap/ng-bootstrap';
import {TaskFormComponent} from '../task-form/task-form.component';
import {TableComponentBase} from '../services/table-component.base';
import {ActivatedRoute, Router} from '@angular/router';

@Component({
  selector: 'app-userstory-table',
  templateUrl: './task-table.component.html',
  styleUrls: ['./task-table.component.css']
})
export class TaskTableComponent extends TableComponentBase<ScrumTask> {
  public filterUserstoryId: number | null = null;
  public get filteredItems() {
    if (this.filterUserstoryId === null || isNaN(this.filterUserstoryId)) {
      return this.items;
    }
    return this.items.filter(task => task.userstoryid === this.filterUserstoryId);
  }

  constructor(
    private backendService: BackendService, private modalService: NgbModal,
    private route: ActivatedRoute, private router: Router
  ) {
    super();

    const params = route.snapshot.paramMap;
    if (params.has('userstoryId')) {
      this.filterUserstoryId = parseInt(params.get('userstoryId'));
    }
    if (params.has('id')) {
      const id = parseInt(params.get('id'));
      this.openTaskForm(this.items.find(t => t.id === id));
    }

    backendService.getTasks().subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.items.push(...response.body);
      }
    });
  }

  public deleteTask(task: ScrumTask) {
    this.backendService.deleteTask(task).subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      }
    });
    const index = this.items.indexOf(task);
    if (index !== -1) {
      this.items.splice(index, 1);
    }
  }

  public openTaskForm(editTask?: ScrumTask) {
    const modalRef = this.modalService.open(TaskFormComponent, {
      backdrop: 'static',
      keyboard: true,
    });
    if (editTask === null) {
      modalRef.result.then(result => {
        this.items.push(result);
      });
    }
    modalRef.componentInstance.task = editTask;
  }

  sortById() {
    this.doNumericSort('id', us => us.id);
  }

  sortByTitle() {
    this.doStringSort('title', us => us.title);
  }

  sortByContent() {
    this.doStringSort('content', us => us.content);
  }

  private getNumberForPriority(priority: Priority): number {
    switch (priority) {
      case Priority.High:
        return 2;
      case Priority.Medium:
        return 1;
      case Priority.Low:
        return 0;
    }
  }

  sortByPrio() {
    this.doNumericSort('priority', us => this.getNumberForPriority(us.priority));
  }

  getNumberOfTasks(userstory: ScrumUserstory) {
    return this.items.filter(t => t.userstoryid === userstory.id).length;
  }

  sortByTasks() {
    this.doNumericSort('tasks', us => this.getNumberOfTasks(us));
  }
}
