import {Component} from '@angular/core';
import {BackendService, ScrumTask, ScrumUserstory} from '../services/backend.service';
import {NgbModal} from '@ng-bootstrap/ng-bootstrap';
import {TableComponentBase} from '../services/table-component.base';
import {getNumberForPriority} from '../services/sorting.service';
import {UserstoryFormComponent} from '../userstory-form/userstory-form.component';
import {ActivatedRoute, Router} from '@angular/router';

@Component({
  selector: 'app-userstory-table',
  templateUrl: './userstory-table.component.html',
  styleUrls: ['./userstory-table.component.css']
})
export class UserstoryTableComponent extends TableComponentBase<ScrumUserstory> {
  public tasks: ScrumTask[] = [];

  constructor(
    private backendService: BackendService, private modalService: NgbModal,
    private route: ActivatedRoute, private router: Router
  ) {
    super();

    const params = route.snapshot.paramMap;

    backendService.getUserstories().subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.items.push(...response.body);

        // Wenn  /userstories;id=3  aufgerufen wird, öffne die Userstory mit ID 3
        const userstory = this.findUserstoryById(parseInt(params.get('id')));
        if (userstory !== null) {
          this.openUserstoryForm(userstory);
        }
      }
    });
    backendService.getTasks().subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.tasks.push(...response.body);
      }
    });
  }

  private findUserstoryById(id: number): ScrumUserstory {
    const userstoriesWithId = this.items.filter(us => us.id === id);
    if (userstoriesWithId.length === 0) {
      return null;
    }
    return userstoriesWithId[0];
  }

  public deleteUserstory(userstory: ScrumUserstory) {
    this.backendService.deleteUserstory(userstory).subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      }
    });
    const index = this.items.indexOf(userstory);
    if (index !== -1) {
      this.items.splice(index, 1);
    }
  }

  public openUserstoryForm(editUserstory?: ScrumUserstory) {
    const modalRef = this.modalService.open(UserstoryFormComponent, {
      backdrop: 'static',
      keyboard: true,
    });
    if (editUserstory === null) {
      modalRef.result.then(result => {
        this.items.push(result);
      });
    }
    modalRef.componentInstance.userstory = editUserstory;
  }

  public getNumberOfTasks(userstory: ScrumUserstory) {
    return this.tasks.filter(t => t.userstoryid === userstory.id).length;
  }

  public sortById() {
    this.doNumericSort('id', us => us.id);
  }

  public sortByTitle() {
    this.doStringSort('title', us => us.title);
  }

  public sortByContent() {
    this.doStringSort('content', us => us.content);
  }

  public sortByPrio() {
    this.doNumericSort('priority', us => getNumberForPriority(us.priority));
  }

  public sortByTasks() {
    this.doNumericSort('tasks', us => this.getNumberOfTasks(us));
  }
}
